@@ -1,15 +1,18 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
+# qmpy/analysis/symmetry/spacegroup.py
 
 import os
 import warnings
 import fractions as frac
 import numpy as np
+import logging
 
 from django.db import models
 
+import qmpy.utils as utils
 from routines import *
 
+logger = logging.getLogger(__name__)
+
 class TranslationError(Exception):
     pass
 
@@ -26,6 +29,26 @@ class SpacegroupError(Exception):
     pass
 
 class Translation(models.Model):
+    """
+    A translation operation.
+
+    Relationships:
+        | :mod:`~qmpy.Spacegroup` via spacegroup
+        | :mod:`~qmpy.Operation` via operation
+
+    Attributes:
+        | id
+        | x, y, z: Translation vector. Accessed via `vector`.
+
+    Examples::
+        
+        >>> op = Operation.get('x', 'x+y', 'z-x+1/2')
+        >>> print op.translsation
+        <Translation: 0,0,+1/2>
+        >>> print op.translation.vector
+        array([ 0. ,  0. ,  0.5])
+
+    """
     x = models.FloatField()
     y = models.FloatField()
     z = models.FloatField()
@@ -57,13 +80,38 @@ class Translation(models.Model):
             if t == 0:
                 s = '0'
             elif t < 0:
-                s = '-%s' % (frac.Fraction(str(t)))
+                s = '%s' % (frac.Fraction(str(t)))
             else:
                 s = '+%s' % (frac.Fraction(str(t)))
             ops.append(s)
         return ','.join(ops)
 
 class Rotation(models.Model):
+    """
+    A rotation operation.
+
+    Relationships:
+        | :mod:`~qmpy.Spacegroup` via spacegroup
+        | :mod:`~qmpy.Operation` via operation
+
+    Attributes:
+        | id
+        | a11, a12, a13
+        | a21, a22, a23
+        | a31, a32, a33: Rotation matrix. Accessed via `matrix`.
+
+    Examples::
+
+        >>> op = Operation.get('x', 'x+y', 'z-x+1/2')
+        >>> print op.rotation
+        <Rotation: x,x+y,-x+z>
+        >>> print op.rotation.matrix
+        array([[ 1.,  0.,  0.],
+               [ 1.,  1.,  0.],
+               [-1.,  0.,  1.]])
+
+
+    """
     a11 = models.FloatField()
     a12 = models.FloatField()
     a13 = models.FloatField()
@@ -112,7 +160,7 @@ class Rotation(models.Model):
                 if x == 0:
                     continue
                 elif x == 1:
-                    s += indict[i]
+                    s += '+'+indict[i]
                 elif x == -1:
                     s += '-'+indict[i]
                 else:
@@ -122,8 +170,25 @@ class Rotation(models.Model):
         return ','.join(ops)
 
 class Operation(models.Model):
-    rotation = models.ForeignKey(Rotation)
-    translation = models.ForeignKey(Translation)
+    """ A symmetry operation (rotation + translation).
+
+    Relationships:
+        | :mod:`~qmpy.Spacegroup` via spacegroup
+        | :mod:`~qmpy.Rotation` via rotation_set
+        | :mod:`~qmpy.Translation` via translation_set
+
+    Attributes:
+        | id
+
+    Examples::
+        
+        >>> op = Operation.get('x+y-1/2,-z-y+1/2,x-z+1/2')
+        >>> print op
+        <Operation: +x+y+1/2,-y-z+1/2,+x-z+1/2>
+
+    """
+    rotation = models.ForeignKey('Rotation')
+    translation = models.ForeignKey('Translation')
 
     class Meta:
         app_label = 'qmpy'
@@ -131,14 +196,16 @@ class Operation(models.Model):
 
     @classmethod
     def get(cls, value):
-        '''
+        """
         Accepts symmetry operation strings, i.e. "+x, x+1/2, x+y-z" or a tuple
         of rotation matrix and translation vector. 
 
-        Example:
-        >>> Operation.get("x,y,-y")
-        >>> Operation.get(( rot, trans ))
-        '''
+        Example::
+
+            >>> Operation.get("x,y,-y")
+            >>> Operation.get(( rot, trans ))
+
+        """
         if isinstance(value, basestring):
             rot, trans = parse_sitesym(value)
         elif isinstance(value, tuple):
@@ -161,7 +228,7 @@ class Operation(models.Model):
                 if x == 0:
                     continue
                 elif x == 1:
-                    s += indict[i]
+                    s += '+'+indict[i]
                 elif x == -1:
                     s += '-'+indict[i]
                 else:
@@ -179,6 +246,21 @@ class Operation(models.Model):
 
 
 class WyckoffSite(models.Model):
+    """
+    Base class for a Wyckoff site. (e.g. a "b" site).
+
+    Relationships:
+        | :mod:`~qmpy.Spacegroup` via spacegroup
+        | :mod:`~qmpy.Atom` via atom_set
+        | :mod:`~qmpy.Site` via site_set
+
+    Attributes:
+        | id
+        | symbol: Site symbol
+        | multiplicity: Site multiplicity
+        | x, y, z: Coordinate symbols.
+
+    """
     spacegroup = models.ForeignKey('Spacegroup', related_name='site_set')
     symbol = models.CharField(max_length=1)
     multiplicity = models.IntegerField(blank=True, null=True)
@@ -202,6 +284,26 @@ class WyckoffSite(models.Model):
         return site
 
 class Spacegroup(models.Model):
+    """
+    Base class for a space group.
+
+    Relationships:
+        | :mod:`~qmpy.Structure` via structure_set
+        | :mod:`~qmpy.Translation` via centering_vectors
+        | :mod:`~qmpy.Operation` via operations
+        | :mod:`~qmpy.WyckoffSite` via site_set
+
+    Attributes:
+        | number: Spacegroup #. (primary key)
+        | centrosymmetric: (bool) Is the spacegroup centrosymmetric.
+        | hall: Hall symbol.
+        | hm: Hermann-Mauguin symobl.
+        | lattice_system: Cubic, Hexagonal, Tetragonal, Orthorhombic,
+        |   Monoclinic or Triclinic.
+        | pearson: Pearson symbol
+        | schoenflies: Schoenflies symbol.
+
+    """
     number = models.IntegerField(primary_key=True)
     hm = models.CharField(max_length=30, blank=True, null=True)
     hall = models.CharField(max_length=30, blank=True, null=True)
@@ -210,7 +312,7 @@ class Spacegroup(models.Model):
     operations = models.ManyToManyField(Operation, null=True)
     centering_vectors = models.ManyToManyField(Translation)
     lattice_system = models.CharField(max_length=20)
-    centrosymmetric = models.BooleanField()
+    centrosymmetric = models.BooleanField(default=False)
 
     _sym_ops = None
     _rots = None
@@ -226,8 +328,17 @@ class Spacegroup(models.Model):
             op.save()
         self.operations = self.sym_ops
 
+    @staticmethod
+    def get(number):
+        try:
+            return Spacegroup.objects.get(number=number)
+        except Spacegroup.DoesNotExist:
+            logger.error('Spacegroup #%s does not exist.' % number)
+            return None
+
     @property
     def sym_ops(self):
+        """List of (rotation, translation) pairs for the spacegroup"""
         if self._sym_ops is None:
             self._sym_ops = [ op for op in self.operations.all() ]
         return self._sym_ops
@@ -238,82 +349,39 @@ class Spacegroup(models.Model):
 
     @property
     def rotations(self):
+        """List of rotation operations for the spacegroup."""
         if self._rots is None:
             self._rots = np.array([ op.rotation.matrix for op in self.sym_ops ])
         return self._rots
 
-    @rotations.setter
-    def rotations(self, rotations):
-        self._rots = rotations
-
     @property
     def translations(self):
+        """List of translation operations for the spacegroup."""
         if self._trans is None:
-            self._trans = np.array([ op.translation.array for op in self.sym_ops ])
+            self._trans = np.array([ op.translation.vector for op in self.sym_ops ])
         return self._trans
 
-    @translations.setter
-    def translations(self, translations):
-        self._trans = translations
-
     def __str__(self):
         return str(self.number)
 
     @property
     def wyckoff_sites(self):
+        """List of WyckoffSites."""
         return self.site_set.all().order_by('symbol')
 
     @property
     def symbol(self):
+        """Returns the Hermann-Mauguin symbol for the spacegroup"""
         return self.hm
 
-    def equivalent_sites(self, structure, symprec=1e-3):
-        sites = []
-        for atom in structure.uniq_atoms:
-            for rot, trans in zip(self.rotations, self.translations):
-                site = np.dot(rot, atom.coord) + trans
-                site %= 1.0
-                a = atom.copy()
-                a.coord = site
-                if not any( s == a for s in sites ):
-                    sites.append(a)
-        return sites
-
-    def symmetry_normalised_sites(self, scaled_positions):
-        """Returns an array of same size as *scaled_positions*,
-        containing the corresponding symmetry-equivalent sites within
-        the unit cell of lowest indices.
-
-        Example:
-
-        >>> from ase.lattice.spacegroup import Spacegroup
-        >>> sg = Spacegroup(225)  # fcc
-        >>> sg.symmetry_normalised_sites([[0.0, 0.5, 0.5], [1.0, 1.0, 0.0]])
-        array([[ 0.,  0.,  0.],
-               [ 0.,  0.,  0.]])
-        """
-        scaled = np.array(scaled_positions, ndmin=2)
-        normalised = np.empty(scaled.shape, np.float)
-        for i, pos in enumerate(scaled):
-            sympos = np.dot(self.rotations, pos) + self.translations
-            sympos %= 1.0
-            sympos %= 1.0
-            j = np.lexsort(sympos.T)[0]
-            normalised[i,:] = sympos[j]
-        return normalised
-
-    def unique_sites(self, structure, symprec=1e-3, output_mask=False):
-        scaled = np.array(structure.scaled_coords, ndmin=2)
-        symnorm = self.symmetry_normalised_sites(scaled)
-        perm = np.lexsort(symnorm.T)
-        iperm = perm.argsort()
-        xmask = np.abs(np.diff(symnorm[perm], axis=0)).max(axis=1) > symprec
-        mask = np.concatenate(([True], xmask))
-        imask = mask[iperm]
-        if output_mask:
-            return [ a for a, i in zip(structure.atoms, imask) if i ], imask
-        else:
-            return [ a for a, i in zip(structure.atoms, imask) if i ]
+    def equivalent_sites(self, point, tol=1e-3):
+        equiv = []
+        for rot, trans in zip(self.rotations, self.translations):
+            new = utils.wrap(np.dot(rot, point) + trans)
+            if not any([ all([ abs(o-n) < tol for o,n in zip(old, new)])
+                                              for old in equiv]):
+                equiv.append(new)
+        return equiv
 
     @property
     def n_sym_ops(self):
@@ -324,5 +392,6 @@ class Spacegroup(models.Model):
         return self.site_set.count()
 
     def get_site(self, symbol):
+        """Gets WyckoffSite by symbol."""
         symol = symbol.strip('0123456789')
         return self.site_set.get(symbol__exact=symbol)
