@@ -9,21 +9,28 @@ import gzip
 import numpy as np
 import numpy.linalg
 import logging
+from collections import defaultdict
 from os.path import exists, isfile, isdir
 
 from django.db import models
+from django.db import transaction
 
-from qmpy.materials.structure import *
-from qmpy.io.poscar import write_poscar, read_poscar
-from potential import *
-from qmpy.data import chem_pots
-from qmpy.db.custom import DictField, NumpyArrayField
+import qmpy
+import qmpy.materials.structure as strx
+import qmpy.io.poscar as poscar
+import potential as pot
+import qmpy.materials.formation_energy as fe
 import qmpy.utils as utils
 import qmpy.db.custom as cdb
-from qmpy.configuration.vasp_settings import *
 import qmpy.analysis.thermodynamics as thermo
 import dos
-from qmpy.data.meta_data import MetaData
+from qmpy.data import chem_pots
+from qmpy.materials.atom import Atom, Site
+from qmpy.utils import *
+from qmpy.data.meta_data import MetaData, add_meta_data
+from qmpy.materials.element import Element
+from qmpy.db.custom import DictField, NumpyArrayField
+from qmpy.configuration.vasp_settings import *
 
 logger = logging.getLogger(__name__)
 
@@ -45,10 +52,49 @@ def vasp_format(key, value):
     return ' %s = %s' % (key.upper(), value_formatter(value))
 
 class VaspError(Exception):
-    '''General problem with vasp calculation.'''
+    """General problem with vasp calculation."""
 
 @add_meta_data('error')
+@add_meta_data('warning')
 class Calculation(models.Model):
+    """
+    Base class for storing a VASP calculation.
+
+    Relationships:
+        | :mod:`~qmpy.Composition` via composition
+        | :mod:`~qmpy.DOS` via dos
+        | :mod:`~qmpy.Structure` via input. Input structure.
+        | :mod:`~qmpy.Structure` via output. Resulting structure.
+        | :mod:`~qmpy.Element` via element_set.
+        | :mod:`~qmpy.Potential` via potential_set.
+        | :mod:`~qmpy.Hubbard` via hubbard_set.
+        | :mod:`~qmpy.Entry` via entry.
+        | :mod:`~qmpy.Fit` via fit. Reference energy sets that have been fit using
+        |   this calculation.
+        | :mod:`~qmpy.FormationEnergy` via formationenergy_set. Formation
+        |   energies computed from this calculation, for different choices of
+        |   fit sets.
+        | :mod:`~qmpy.MetaData` via meta_data
+
+    Attributes:
+        | id
+        | label: key for entry.calculations dict.
+        | attempt: # of this attempt at a calculation.
+        | band_gap: Energy gap occupied by the fermi energy.
+        | configuration: Type of calculation (module).
+        | converged: Did the calculation converge electronically and ionically.
+        | energy: Total energy (eV/UC)
+        | energy_pa: Energy per atom (eV/atom)
+        | irreducible_kpoints: # of irreducible k-points.
+        | magmom: Total magnetic moment (mu_b)
+        | magmom_pa: Magnetic moment per atom. (mu_b/atom)
+        | natoms: # of atoms in the input.
+        | nsteps: # of ionic steps.
+        | path: Calculation path.
+        | runtime: Runtime in seconds.
+        | settings: dictionary of VASP settings.
+
+    """
     #= labeling =#
     configuration = models.CharField(db_index=True, max_length=15, 
             null=True, blank=True)
@@ -63,7 +109,7 @@ class Calculation(models.Model):
     natoms = models.IntegerField(blank=True, null=True)
 
     #= inputs =#
-    input = models.ForeignKey(Structure, 
+    input = models.ForeignKey(strx.Structure, 
                               related_name='calculated',
                               null=True, blank=True)
     hubbard_set = models.ManyToManyField('Hubbard')
@@ -71,7 +117,7 @@ class Calculation(models.Model):
     settings = DictField(blank=True, null=True)
     
     #= outputs =#
-    output = models.ForeignKey(Structure, 
+    output = models.ForeignKey(strx.Structure, 
                                related_name='source',
                                null=True, blank=True)
 
@@ -89,14 +135,11 @@ class Calculation(models.Model):
     converged = models.NullBooleanField(null=True)
     runtime = models.FloatField(blank=True, null=True)
 
-    #= other thermodata =#
-    delta_e = models.FloatField(blank=True, null=True)
-    stability = models.FloatField(blank=True, null=True)
-
     #= Non-stored values =#
     outcar = None
     kpoints = None
     occupations = None
+    formation = None
 
     class Meta:
         app_label = 'qmpy'
@@ -117,27 +160,31 @@ class Calculation(models.Model):
             return 'Blank'
         return retval
 
-    # django overrides
+    @transaction.atomic
     def save(self, *args, **kwargs):
         if self.output is not None:
-            self.output.entry = self.entry
+            if self.entry:
+                self.output.entry = self.entry
             self.output.save()
-            self.output_id = self.output.id
+            self.output = self.output
             self.composition = self.output.composition
         if self.input is not None:
-            self.input.entry = self.entry
+            if self.entry:
+                self.input.entry = self.entry
             self.input.save()
-            self.input_id = self.input.id
+            self.input = self.input
             self.composition = self.input.composition
         if self.dos is not None:
             self.dos.entry = self.entry
             self.dos.save()
-            self.dos_id = self.dos.id
+            self.dos = self.dos
         super(Calculation, self).save(*args, **kwargs)
         self.hubbard_set = self.hubbards
         self.potential_set = self.potentials
         self.element_set = self.elements
         self.meta_data = self.error_objects
+        if self.formation:
+            self.formationenergy_set.add(self.formation)
 
     # django caching
     _potentials = None
@@ -199,6 +246,18 @@ class Calculation(models.Model):
                     hcomp[h] += 1
         return dict(hcomp.items())
 
+    @property
+    def true_comp(self):
+        comp = defaultdict(int)
+        for c, v in self.comp.items():
+            if self.hubbard_set.filter(element=c).exists():
+                h = self.hubbard_set.get(element=c)
+                if h:
+                    comp['%s_%s' % (c, h.u)] += v
+                    continue
+            comp[c] += v
+        return dict(comp)
+
     @property
     def unit_comp(self):
         return unit_comp(self.comp)
@@ -210,7 +269,7 @@ class Calculation(models.Model):
     #= input files as strings =#
     @property
     def POSCAR(self):
-        return write_poscar(self.input)
+        return poscar.write(self.input)
 
     @property
     def INCAR(self):
@@ -244,10 +303,16 @@ class Calculation(models.Model):
         p = thermo.Phase(energy=self.delta_e,
                 composition=parse_comp(self.composition_id),
                 description=str(self.input.spacegroup),
+                stability=self.stability,
                 per_atom=True)
         p.id = self.id
         return p
 
+    def calculate_stability(self, fit='standard'):
+        from qmpy.analysis.thermodynamics import PhaseSpace
+        ps = PhaseSpace(self.input.comp)
+        ps.compute_stabilities()
+
     def get_incar(self):
         s = dict((k.lower(), v) for k, v in self.settings.items() if not k in
                 ['gamma', 'kppra', 'scale_encut', 'potentials', 'hubbards'])
@@ -259,9 +324,9 @@ class Calculation(models.Model):
 
         if self.MAGMOMS and not 'ispin' in s:
             s['ispin'] = 2
-        incar += ' ISPIN = %d\n' % s.pop('ispin', 1)
+        incar += '  ISPIN = %d\n' % s.pop('ispin', 1)
         if self.MAGMOMS:
-            incar += self.MAGMOMS
+            incar += self.MAGMOMS+'\n'
 
         if  any(hub for hub in self.hubbards):
             incar += '\n#= LDA+U Fields =#\n'
@@ -336,7 +401,7 @@ class Calculation(models.Model):
                 assert len(ldauls) == len(atom_types)
                 assert len(ldauus) == len(atom_types)
                 for elt, u, l in zip(atom_types, ldauus, ldauls):
-                    hub, created = Hubbard.objects.get_or_create(element_id=elt,
+                    hub, created = pot.Hubbard.objects.get_or_create(element_id=elt,
                             u=u, l=l)
                     self.hubbards.append(hub)
             if magmoms:
@@ -354,21 +419,7 @@ class Calculation(models.Model):
         self.settings = settings
 
     def get_kpoints(self):
-        n = float(len(self.input.atoms))
-        recs = self.input.reciprocal_lattice
-        rec_mags = [ norm(recs[0]), norm(recs[1]), norm(recs[2])]
-        r0 = min(rec_mags)
-        refr = np.array([ np.round(r/r0, 4) for r in rec_mags ])
-        scale = 1.0
-        kpts = np.ones(3)
-        kppra = self.settings.get('kppra', 8000)
-        while n*np.product(kpts) < kppra:
-            prev_kpts = kpts.copy()
-            refk = np.array(np.ones(3)*refr)*scale
-            kpts = np.array(map(np.ceil, refk))
-            scale += 1
-        if kppra - np.product(prev_kpts)*n < np.product(kpts)*n - kppra:
-            kpts = prev_kpts.copy()
+        kpts = self.input.get_kpoint_mesh(self.settings.get('kppra', 8000))
         if self.settings.get('gamma', True):
             kpoints = 'KPOINTS \n0 \nGamma\n'
         else:
@@ -398,13 +449,13 @@ class Calculation(models.Model):
 
     @POTCAR.setter
     def POTCAR(self, potcar):
-        pots = Potential.read_potcar(potcar)
+        pots = pot.Potential.read_potcar(potcar)
         for pot in pots:
             self.potentials.append(pot)
 
     @POSCAR.setter
     def POSCAR(self, poscar):
-        self.input = read_poscar(poscar)
+        self.input = poscar.read(poscar)
 
     #= Results Reading =#
 
@@ -412,19 +463,20 @@ class Calculation(models.Model):
         raise NotImplementedError
 
     def get_outcar(self):
-        '''
+        """
         Sets the calculations outcar attribute to a list of lines from the
         outcar. 
 
-        Example:
-        >>> calc = Calculation.read('calculation_path')
-        >>> print calc.outcar
-        None
-        >>> calc.get_outcar()
-        >>> len(calc.outcar)
-        12345L
+        Examples::
+
+            >>> calc = Calculation.read('calculation_path')
+            >>> print calc.outcar
+            None
+            >>> calc.get_outcar()
+            >>> len(calc.outcar)
+            12345L
         
-        '''
+        """
         if not self.outcar is None:
             return self.outcar
         if not exists(self.path):
@@ -436,16 +488,28 @@ class Calculation(models.Model):
         else:
             raise VaspError('No such file exists')
 
+    def read_runtime(self):
+        self.get_outcar()
+        runtime = 0
+        for line in self.outcar:
+            if 'LOOP+' in line:
+                if not len(line.split()) == 7:
+                    continue
+                runtime += ffloat(line.split()[-1])
+        self.runtime = runtime
+        return runtime
+
     def read_energies(self):
-        '''
+        """
         Returns a numpy.ndarray of energies over all ionic steps.
 
-        Example:
-        >>> calc = Calculation.read('calculation_path')
-        >>> calc.read_energies()
-        array([-12.415236, -12.416596, -12.416927])
+        Examples::
+
+            >>> calc = Calculation.read('calculation_path')
+            >>> calc.read_energies()
+            array([-12.415236, -12.416596, -12.416927])
         
-        '''        
+        """        
         self.get_outcar()
         energies = []
         for line in self.outcar:
@@ -455,7 +519,7 @@ class Calculation(models.Model):
         return self.energies
 
     def read_natoms(self):
-        '''Reads the number of atoms, and assigns the value to natoms.'''
+        """Reads the number of atoms, and assigns the value to natoms."""
         self.get_outcar()
         for line in self.outcar:
             if 'NIONS' in line:
@@ -463,26 +527,28 @@ class Calculation(models.Model):
                 return self.natoms
 
     def read_n_ionic(self):
-        '''Reads the number of ionic steps, and assigns the value to nsteps.'''
+        """Reads the number of ionic steps, and assigns the value to nsteps."""
         self.get_outcar()
         self.nsteps = len([ l for l in self.outcar if 'free  energy' in l ])
         return self.nsteps
 
     def read_input_structure(self):
         if os.path.exists(self.path+'/POSCAR'):
-            self.input = read_poscar(self.path+'/POSCAR')
+            self.input = poscar.read(self.path+'/POSCAR')
             self.input.entry = self.entry
 
     def read_elements(self):
-        '''
+        """
         Reads the elements of the atoms in the structure. Returned as a list of
         atoms of shape (natoms,). 
 
-        Example:
-        >>> calc = Calculation.read('path_to_calculation')
-        >>> calc.read_elements()
-        ['Fe', 'Fe', 'O', 'O', 'O']
-        '''
+        Examples::
+
+            >>> calc = Calculation.read('path_to_calculation')
+            >>> calc.read_elements()
+            ['Fe', 'Fe', 'O', 'O', 'O']
+
+        """
         self.get_outcar()
         elt_list = []
         elements = []
@@ -502,54 +568,48 @@ class Calculation(models.Model):
         return self.elements
 
     def read_lattice_vectors(self):
-        '''
+        """
         Reads and returns a numpy ndarray of lattice vectors for every ionic 
         step of the calculation.
 
-        Example:
-        >>> calc = Calculation.read('path_to_calculation')
-        >>> calc.read_lattice_vectors()
-        array([[[ 4.17458823,  0.        ,  0.        ],
-                [-2.08729412,  3.61529939,  0.        ],
-                [ 0.        ,  0.        ,  5.20720757]],
+        Examples::
 
-               [[ 4.17458823,  0.        ,  0.        ],
-                [-2.08729412,  3.61529939,  0.        ],
-                [ 0.        ,  0.        ,  5.20720757]],
+            >>> path = 'analysis/vasp/files/magnetic/standard'
+            >>> calc = Calculation.read(INSTALL_PATH+'/'+path)
+            >>> calc.read_lattice_vectors()
+            array([[[ 5.707918,  0.      ,  0.      ],
+                    [ 0.      ,  5.707918,  0.      ],
+                    [ 0.      ,  0.      ,  7.408951]],
+                   [[ 5.707918,  0.      ,  0.      ],
+                    [ 0.      ,  5.707918,  0.      ],
+                    [ 0.      ,  0.      ,  7.408951]]])
 
-               [[ 4.15950113,  0.        ,  0.        ],
-                [-2.07975057,  3.60223358,  0.        ],
-                [ 0.        ,  0.        ,  5.18670694]],
-
-               [[ 4.16254664,  0.        ,  0.        ],
-                [-2.08127332,  3.60487107,  0.        ],
-                [ 0.        ,  0.        ,  5.18394359]]])
-
-        '''
+        """
         self.get_outcar()
         lattice_vectors = []
         for i, line in enumerate(self.outcar):
             if 'direct lattice vectors' in line:
                 tlv = []
                 for n in range(3):
-                    vector = self.outcar[i+n+1].split()[:3]
-                    tlv.append(map(float, vector))
+                    tlv.append(read_fortran_array(self.outcar[i+n+1], 6)[:3])
                 lattice_vectors.append(tlv)
         return np.array(lattice_vectors)
 
     def read_charges(self):
-        '''
+        """
         Reads and returns VASP's calculated charges for each atom. Returns the
         RAW charge, not NET charge.
         
-        Example:
-        >>> calc = Calculation.read('path_to_calculation')
-        >>> calc.read_charges()
+        Examples::
 
-        '''
+            >>> calc = Calculation.read('path_to_calculation')
+            >>> calc.read_charges()
+
+        """
         self.get_outcar()
         self.read_natoms()
         self.read_n_ionic()
+        self.read_runtime()
         if self.settings is None:
             self.read_outcar_settings()
         if not self.settings['lorbit'] == 11:
@@ -562,7 +622,7 @@ class Calculation(models.Model):
                 for i in range(self.natoms):
                     chgs.append(float(self.outcar[n+4+i].split()[-1]))
                 charges.append(chgs)
-        return np.array(charges[:-1])
+        return np.array(charges)
 
     def read_magmoms(self):
         self.get_outcar()
@@ -570,7 +630,7 @@ class Calculation(models.Model):
         self.read_n_ionic()
         if self.settings is None:
             self.read_outcar_settings()
-        if self.settings['ispin'] == 1 or self.settings['lorbit'] != 11:
+        if self.settings['ispin'] == 1:
             return np.array([[0]*self.natoms]*self.nsteps)
 
         magmoms = []
@@ -583,7 +643,9 @@ class Calculation(models.Model):
             if 'number of electron' in line:
                 if 'magnetization' in line:
                     self.magmom = float(line.split()[-1])
-        return magmoms[:-1]
+        if self.settings['lorbit'] != 11:
+            return np.array([[0]*self.natoms]*self.nsteps)
+        return magmoms
 
     def read_forces(self):
         self.get_outcar()
@@ -672,27 +734,25 @@ class Calculation(models.Model):
         self.bands = np.array(bands)
 
     def read_outcar_results(self):
-        try:
-            self.read_natoms()
-            self.read_convergence()
-            elts = self.read_elements()
-            energies = self.read_energies()
-            lattice_vectors = self.read_lattice_vectors()
-            stresses = self.read_stresses()
-            positions = self.read_positions()
-            forces = self.read_forces()
-            magmoms = self.read_magmoms()
-            charges = self.read_charges()
-        except:
-            self.add_error('incomplete')
+        self.read_natoms()
+        self.read_convergence()
+        elts = self.read_elements()
+        energies = self.read_energies()
+        lattice_vectors = self.read_lattice_vectors()
+        stresses = self.read_stresses()
+        positions = self.read_positions()
+        forces = self.read_forces()
+        magmoms = self.read_magmoms()
+        charges = self.read_charges()
 
         if len(self.energies) > 0:
             self.energy = self.energies[-1]
             self.energy_pa = self.energy/self.natoms
         if not self.magmom is None:
             self.magmom_pa = self.magmom/self.natoms
-        if self.nsteps > 1:
-            output = Structure()
+
+        if self.nsteps > 0:
+            output = strx.Structure()
             output.total_energy = energies[-1]
             output.cell = lattice_vectors[-1]
             output.stresses = stresses[-1]
@@ -709,8 +769,7 @@ class Calculation(models.Model):
                 atoms.append(a)
             output.atoms = atoms
             self.output = output
-        else:
-            self.output = self.input
+            self.output.set_label(self.label)
 
     def read_convergence(self):
         self.get_outcar()
@@ -871,7 +930,7 @@ class Calculation(models.Model):
         elts = [ p['name'].split('_')[0] for p in settings['potentials'] ]
         if any([ len(s) > 1 for s in [xcs, uss, paws]]):
             raise VaspError('Not all potentials are of the same type')
-        self.potentials = Potential.objects.filter(us=uss[0], 
+        self.potentials = pot.Potential.objects.filter(us=uss[0], 
                                               xc=xcs[0], 
                                               paw=paws[0],
                                               name__in=pot_names)
@@ -882,7 +941,7 @@ class Calculation(models.Model):
             for elt, l, u in zip(elts, 
                                  settings['ldauls'], 
                                  settings['ldauus']):
-                self.hubbards.append(Hubbard.get(elt, u=u, l=l))
+                self.hubbards.append(pot.Hubbard.get(elt, u=u, l=l))
         self.settings = settings
 
     def read_stdout(self, filename='stdout.txt'):
@@ -891,35 +950,59 @@ class Calculation(models.Model):
         stdout = open('%s/%s' % (self.path, filename)).read()
         errors = []
         if 'Error reading item' in stdout:
-            errors.append('input_error')
+            self.add_error('input_error')
         if 'ZPOTRF' in stdout:
-            errors.append('zpotrf')
+            self.add_error('zpotrf')
         if 'SGRCON' in stdout:
-            errors.append('sgrcon')
+            self.add_error('sgrcon')
         if 'INVGRP' in stdout:
-            errors.append('invgrp')
+            self.add_error('invgrp')
         if 'BRIONS problems: POTIM should be increased' in stdout:
-            errors.append('brions')
+            self.add_error('brions')
+        if 'TOO FEW BANDS' in stdout:
+            self.add_error('bands')
         if 'FEXCF' in stdout:
-            errors.append('fexcf')
+            self.add_error('fexcf')
         if 'FEXCP' in stdout:
-            errors.append('fexcp')
+            self.add_error('fexcp')
         if 'PRICEL' in stdout:
-            errors.append('pricel')
+            self.add_error('pricel')
         if 'EDDDAV' in stdout:
-            errors.append('edddav')
+            self.add_error('edddav')
         if 'Sub-Space-Matrix is not hermitian in DAV' in stdout:
-            errors.append('hermitian')
+            self.add_error('hermitian')
         if 'IBZKPT' in stdout:
-            errors.append('IBZKPT error')
+            self.add_warning('IBZKPT error')
         if 'BRMIX: very serious problems' in stdout:
-            errors.append('brmix')
-        return errors
+            self.add_error('brmix')
+        return self.errors
+
+    def read_outcar_started(self):
+        self.get_outcar()
+        if len(self.outcar) < 5:
+            return False
+        found_inputs = [False, False, False, False]
+        for line in self.outcar:
+            if 'INCAR:' in line:
+                found_inputs[0] = True
+            if 'POTCAR:' in line: 
+                found_inputs[1] = True
+            if 'KPOINTS:' in line:
+                found_inputs[2] = True
+            if 'POSCAR:' in line:
+                found_inputs[3] = True
+            if all(found_inputs):
+                break
+        if not all(found_inputs):
+            return False
+        return True
 
     def read_outcar(self):
+        self.get_outcar()
+        if not self.read_outcar_started():
+            return
         if self.input is None:
             self.read_input_structure()
-        self.get_outcar()
         self.read_outcar_settings()
         self.read_outcar_results()
         self.read_n_ionic()
@@ -958,14 +1041,17 @@ class Calculation(models.Model):
 
     @staticmethod
     def read(path):
-        '''
+        """
         Reads the outcar specified by the objects path. Populates input field
         values, as well as outputs, in addition to finding errors and
         confirming convergence.
 
-        Example:
-        >>> calc = Calculation.read('/path/to/calculation')
-        '''
+        Examples:
+
+            >>> path = '/analysis/vasp/files/normal/standard/'
+            >>> calc = Calculation.read(INSTALL_PATH+path)
+
+        """
         path = os.path.abspath(path)
         existing = Calculation.objects.filter(path=path)
         if existing.count() > 1:
@@ -976,35 +1062,54 @@ class Calculation(models.Model):
         calc = Calculation(path=path)
         if calc.input is None:
             calc.read_input_structure()
-        calc.label = os.path.basename(calc.path)
+        calc.set_label(os.path.basename(calc.path))
         calc.read_outcar()
         calc.read_stdout()
         if calc.converged:
             calc.read_doscar()
-        #except Exception, err:
-        #    calc.errors.append('exception: %s' % err)
+        if not calc.output is None:
+            calc.output.set_label(calc.label)
         return calc
 
+    @staticmethod
+    def read_tree(path):
+        path = os.path.abspath(path)
+        contents = os.listdir(path)
+        prev_calcs = [ f for f in contents if 
+                       os.path.isdir('%s/%s' % (path, f)) ]
+        prev_calcs = sorted(prev_calcs, key=lambda x: -int(x.split('_')[0]))
+
+        calcs = [Calculation.read(path)]
+        for i, calc in enumerate(prev_calcs):
+            c = Calculation.read('%s/%s' % (path, calc))
+            c.set_label('%s_%s' % (calcs[0].label, calc.split('_')[0]))
+            calcs[-1].input = c.output
+            calcs.append(c)
+        return calcs
+
     def address_errors(self):
         errors = self.errors
-        if not errors:
+        if not errors or errors == ['found no errors']:
             logger.info('Found no errors')
             return self
         
         if self.label == '':
-            self.label = os.path.basename(self.path)
+            self.set_label(os.path.basename(self.path))
         new_calc = self.copy()
-        self.label = self.label + '_%d' % self.attempt
+        new_calc.set_label(self.label)
+        self.set_label(self.label + '_%d' % self.attempt)
         new_calc.attempt += 1
+        if new_calc.attempt > 5:
+            new_calc.add_error('attempts')
 
         for err in errors:
             if err in ['duplicate','partial']:
                 continue
             elif err == 'convergence':
-                if self.nsteps > 1:
+                if not self.output is None:
                     new_calc.remove_error('convergence')
                     new_calc.input = self.output
-                    new_calc.input.label = self.label
+                    new_calc.input.set_label(self.label)
             elif err == 'electronic_convergence':
                 new_calc.fix_electronic_convergence()
             elif err == 'doscar_exc':
@@ -1015,62 +1120,52 @@ class Calculation(models.Model):
                 new_calc.fix_dav()
             elif err == 'errrmm':
                 new_calc.fix_rmm()
-            elif err == 'sgrcon':
-                new_calc.fix_sgrcon()
-            elif err == 'zpotrf':
-                new_calc.fix_zpotrf()
-            elif err == 'invgrp':
-                new_calc.fix_invgrp()
-            elif err == 'exception':
-                new_calc.clean_start()
-            elif err == 'fexcf':
-                new_calc.fix_fexcf()
-            elif err == 'fexcp':
-                new_calc.fix_fexcp()
             elif err == 'brions':
                 new_calc.fix_brions()
             elif err == 'brmix':
                 new_calc.fix_brmix()
+            elif err in [ 'zpotrf', 'fexcp', 'fexcf']:
+                new_calc.reduce_potim()
             elif err in ['pricel', 'invgrp', 'sgrcon']:
                 new_calc.increase_symprec()
             elif err == 'hermitian':
                 new_calc.fix_hermitian()
             else:
                 print err
-
-        new_calc.set_magmoms(self.settings.get('magnetism', 'ferro'))
         return new_calc
 
-    #= File handling =#
-
     def compress(self):
         for file in os.listdir(self.path):
             if file in ['OUTCAR', 'CHGCAR', 'CHG', 'PROCAR', 'LOCPOT', 'ELFCAR']:
                 os.system('gzip -f %s' % self.path+'/'+file)
 
     def copy(self):
-        calc = copy.deepcopy(self)
-        calc.id = None
-        calc.label = ''
-        if self.input is not None:
-            calc.input = self.input
-        if self.output is not None:
-            calc.output = self.output
-        calc.potentials = self.potentials
-        calc.hubbards = self.hubbards
-        return calc
+        new = Calculation()
+        new.entry = self.entry
+        new.label = ''
+        new.input = self.input
+        new.output = self.output
+        new.dos = self.dos
+        new.path = self.path
+        new.potentials = self.potentials
+        new.hubbards = self.hubbards
+        new.settings = self.settings
+        new.attempt = self.attempt 
+        new.composition = self.composition
+        return new
 
     def move(self, path):
         path = os.path.abspath(path)
-        os.system('mkdir %s &> /dev/null' % path)
-        os.system('cp %s/* %s &> /dev/null' % (self.path, path))
-        os.system('rm %s/* &> /dev/null' % self.path)
+        os.system('mkdir %s 2> /dev/null' % path)
+        os.system('cp %s/* %s 2> /dev/null' % (self.path, path))
+        os.system('rm %s/* 2> /dev/null' % self.path)
         self.path = path
         self.save()
 
     def backup(self):
         new_dir = '%s_' % self.attempt
         new_dir += '_'.join(self.errors)
+        new_dir = new_dir.replace(' ','')
         logger.info('backing up %s to %s' % (self.path, new_dir))
         self.move(self.path+'/'+new_dir)
 
@@ -1091,11 +1186,6 @@ class Calculation(models.Model):
                               'algo':'normal'})
         self.remove_error('brmix')
 
-    def fix_fexcf(self):
-        self.settings.update({'potim':0.1,
-                              'algo':'normal'})
-        self.remove_error('fexcf')
-
     def fix_electronic_convergence(self):
         if not self.settings.get('algo') == 'normal':
             self.settings['algo'] = 'normal'
@@ -1111,13 +1201,15 @@ class Calculation(models.Model):
         self.settings['potim'] *= 2
         self.remove_error('brions')
 
-    def fix_zpotrf(self):
+    def reduce_potim(self):
         self.settings.update({'algo':'normal',
                               'potim':0.1})
         self.remove_error('zpotrf')
+        self.remove_error('fexcp')
+        self.remove_error('fexcf')
 
     def fix_bands(self):
-        self.settings['nbands'] *= int(np.ceil(1.15))
+        self.settings['nbands'] = int(np.ceil(self.settings['nbands']*1.5))
         self.errors = []
 
     def fix_dav(self):
@@ -1150,7 +1242,7 @@ class Calculation(models.Model):
     #### calculation management
 
     def write(self):
-        os.system('mkdir %s &> /dev/null' % self.path)
+        os.system('mkdir %s 2> /dev/null' % self.path)
         poscar = open(self.path+'/POSCAR','w')
         potcar = open(self.path+'/POTCAR','w')
         incar = open(self.path+'/INCAR','w')
@@ -1166,7 +1258,7 @@ class Calculation(models.Model):
 
     @property
     def estimate(self):
-        return 24*8*3600
+        return 48*8*3600
 
     @property
     def instructions(self):
@@ -1193,6 +1285,11 @@ class Calculation(models.Model):
                 'serial':True})
         return instruction
 
+    def set_label(self, label):
+        self.label = label
+        if not self.entry is None:
+            self.entry.calculations[label] = self
+
     def set_hubbards(self, convention='wang'):
         hubs = HUBBARDS.get(convention, {})
         elts = set( k[0] for k in hubs.keys() )
@@ -1212,12 +1309,12 @@ class Calculation(models.Model):
             for hub in hubs:
                 if ( atom.element_id == hub[0] and 
                         hub[2] in [ None, atom.ox ]):
-                    self.hubbards.append(Hubbard.get(
+                    self.hubbards.append(pot.Hubbard.get(
                         hub[0], lig=hub[1], ox=hub[2],
                         u=hubs[hub]['U'], l=hubs[hub]['L']))
                     break
             else:
-                self.hubbards.append(Hubbard.get(atom.element_id))
+                self.hubbards.append(pot.Hubbard.get(atom.element_id))
         self.hubbards = list(set(self.hubbards))
 
     def set_potentials(self, choice='vasp_rec', distinct_by_ox=False):
@@ -1225,7 +1322,7 @@ class Calculation(models.Model):
             if len(self.potentials) == len(choice):
                 return
         pot_set = POTENTIALS[choice]
-        potentials = Potential.objects.filter(xc=pot_set['xc'],
+        potentials = pot.Potential.objects.filter(xc=pot_set['xc'],
                                               gw=pot_set['gw'],
                                               us=pot_set['us'],
                                               paw=pot_set['paw'])
@@ -1238,12 +1335,30 @@ class Calculation(models.Model):
         if any(self.input.magmoms):
             self.ispin = 2
 
+    @property
+    def volume(self):
+        if self.output:
+            return self.output.get_volume()
+        elif self.input:
+            return self.input.get_volume()
+
+    @property
+    def volume_pa(self):
+        if self.volume is None:
+            return
+        return self.volume/len(self.output)
+
     def compute_formation(self, reference='standard'):
-        #if not self.label == 'standard':
-        #    print 'WARNING: This formation energy is with respect to\
-        #            innappropriate reference states and cannot be trusted!'
         if not self.converged:
             return None
+        if len(self.input.comp) == 1:
+            e = self.input.composition.total_energy
+            formation = fe.FormationEnergy.get(self, fit=reference)
+            formation.delta_e = self.energy_pa - e
+            formation.composition = self.input.composition
+            formation.entry = self.entry
+            self.formation = formation
+            return formation
         hub_mus = chem_pots[reference]['hubbards']
         if self.hub_comp and reference == 'nothing':
             return
@@ -1251,8 +1366,12 @@ class Calculation(models.Model):
         adjust = 0
         adjust -= sum([ hub_mus.get(k.key, 0)*v for k,v in self.hub_comp.items() ])
         adjust -= sum([ elt_mus[k]*v for k,v in self.comp.items() ])
-        self.delta_e = ( self.energy + adjust ) / self.natoms
-        return self.delta_e
+        formation = fe.FormationEnergy.get(self, fit=reference)
+        formation.delta_e = ( self.energy + adjust ) / self.natoms
+        formation.composition = self.input.composition
+        formation.entry = self.entry
+        self.formation = formation
+        return formation
 
     @staticmethod
     def setup(input=None, configuration='standard', path=None, entry=None,
@@ -1279,6 +1398,8 @@ class Calculation(models.Model):
         if configuration not in VASP_SETTINGS:
             raise NotImplementedError('That configuration does not exist!')
 
+        calc.input.make_primitive()
+
         settings = {}
         if calc.input.natoms > 20:
             settings['lreal'] = 'auto'
@@ -1293,13 +1414,13 @@ class Calculation(models.Model):
             calc.encut = int(settings.get('scale_encut')*enmax)
 
         calc.settings = settings
+        if calc.input.natoms >= 10:
+            calc.settings.update({
+                'ncore': 4, 
+                'lscalu': False,
+                'lplane': True})
 
         #= Error handling stuff =#
-        if set(calc.errors) <= set(['attempts', 'ionic_convergence']):
-            calc.remove_error('attempts')
-            calc.remove_error('ionic_convergence')
-
-        # has the calculation been run yet?
         try:
             calc.get_outcar()
         except VaspError:
@@ -1321,6 +1442,11 @@ class Calculation(models.Model):
             return calc
         
         fixed_calc = calc.address_errors()
+        if fixed_calc.errors:
+            # unable to fix errors
+            return fixed_calc
+
+        fixed_calc.set_magmoms(calc.settings.get('magnetism', 'ferro'))
         fixed_calc.clear_results()
         calc.backup()
         fixed_calc.clear_outputs()
