@@ -4,40 +4,93 @@ import subprocess
 import os
 import os.path
 import time
+from collections import defaultdict
+import json
+import logging
 
 from django.db import models
-from django.contrib.auth.models import User
+from django.contrib.auth.models import AbstractUser
 
+import qmpy
 from qmpy.db.custom import DictField
+import queue as queue
+
+logger = logging.getLogger(__name__)
 
 class AllocationError(Exception):
-    '''Problem with the allocation'''
+    """Problem with the allocation"""
 
 class SubmissionError(Exception):
-    '''Failed to submit a job'''
+    """Failed to submit a job"""
+
+class User(AbstractUser):
+    """
+    User model - stores an oqmd users information.
+
+    Relationships:
+        | :mod:`~qmpy.Account` via account_set
+        | :mod:`~qmpy.Allocation` via allocation_set
+        | :mod:`~qmpy.Project` via project_set
+
+    Attributes:
+        | id
+        | username
+        | first_name
+        | last_name
+        | date_joined
+        | is_active
+        | is_staff
+        | is_superuser
+        | last_login
+        | email
+
+    """
+
+    class Meta:
+        app_label = 'qmpy'
+        db_table = 'users'
+
+    @property
+    def running(self):
+        return queue.Job.objects.filter(account__user=self, state=1)
+
+    @classmethod
+    def get(cls, name):
+        try:
+            return cls.objects.get(username=name)
+        except cls.DoesNotExist:
+            return cls(username=name)
 
 class Host(models.Model):
-    '''
+    """
     Host model - stores all host information for a cluster.
 
-    Stored attributes:
-        name
-        ip_address
-        hostname
-        binaries    :   dict of binary names and paths 
-        ppn         :   # of processors per node (int)
-        nodes
-        walltime
-        sub_script
-        sub_text
-        check_queue
-        checked_time
-        running
-        utilization
-    '''
-    name = models.CharField(max_length=20, db_index=True)
-    ip_address = models.CharField(max_length=120)
-    hostname = models.CharField(max_length=120)
+    Relationships:
+        | account
+        | allocation
+
+    Attributes:
+        | name: Primary key.
+        | binaries: dict of label:path pairs for vasp binaries.
+        | check_queue: Path to showq command
+        | checked_time: datetime object for the last time the queue was 
+        |   checked.
+        | hostname: Full host name. 
+        | ip_address: Full ip address.
+        | nodes: Total number of nodes.
+        | ppn: Number of processors per node.
+        | running: dict of PBS_ID:state pairs.
+        | sub_script: Path to qsub command
+        | sub_text: Path to queue file template.
+        | utilization: Number of active cores (based on showq).
+        | walltime: Maximum walltime on the machine.
+        | state: State code. 1=Up, 0=Full (auto-resets to 1 when jobs are
+        |   collected), -1=Down.
+
+    """
+    name = models.CharField(max_length=63, primary_key=True)
+    ip_address = models.IPAddressField(null=True)
+    hostname = models.CharField(max_length=255)
     binaries = DictField()
     ppn = models.IntegerField(default=8)
     nodes = models.IntegerField(default=30)
@@ -47,7 +100,7 @@ class Host(models.Model):
     check_queue = models.CharField(max_length=180,
             default='/usr/local/maui/bin/showq')
     checked_time = models.DateTimeField(default=datetime.min)
-    running = DictField(blank=True, null=True)
+    running = DictField()
     utilization = models.IntegerField(default=0)
     state = models.IntegerField(default=1)
 
@@ -60,11 +113,11 @@ class Host(models.Model):
 
     @classmethod
     def interactive_create(cls):
-        '''
+        """
         Classmethod to create a Host model. Script will ask you questions about
         the host to add, and will return the created Host.
 
-        '''
+        """
         host = {}
         host['name'] = raw_input('Hostname:')
         if cls.objects.filter(name=host['name']).exists():
@@ -84,6 +137,13 @@ class Host(models.Model):
         h = cls(**host)
         h.save()
 
+    @classmethod
+    def get(cls, name):
+        try:
+            return cls.objects.get(name=name)
+        except cls.DoesNotExist:
+            return cls(name=name)
+
     @property
     def accounts(self):
         return list(self.account_set.all())
@@ -108,14 +168,31 @@ class Host(models.Model):
     def percent_utilization(self):
         return 100. * float(self.utilization) / (self.nodes*self.ppn)
 
-    @property
-    def utilization(self):
+    def get_utilization(self):
         util = 0
         for acct in self.account_set.all():
             for job in acct.job_set.filter(state=1):
                 util += job.ncpus
+        self.utilization = util
         return util
 
+    def get_project(self):
+        proj = Project.objects.filter(allocations__host=self, state=1)
+        proj = proj.filter(task__state=0)
+        if proj.exists():
+            return random.choice(list(proj.distinct()))
+
+    def get_tasks(self, project=None):
+        tasks = queue.Task.objects.filter(state=0)
+        if project is None:
+            project = self.get_project()
+            if project is None:
+                return
+        tasks = tasks.filter(project_set=project)
+        tasks = tasks.filter(project_set__allocations__host=self)
+        tasks = tasks.filter(project_set__users__account__host=self)
+        return tasks.order_by('priority')
+
     @property
     def qfile(self):
         return open(self.sub_text).read()
@@ -124,8 +201,8 @@ class Host(models.Model):
         return self.binaries[key]
 
     def check_host(self):
-        '''Pings the host to see if it is online. Returns False if it is
-        offline.'''
+        """Pings the host to see if it is online. Returns False if it is
+        offline."""
         ret = subprocess.call("ping -c 1 -w 1 %s" % self.ip_address,
                 shell=True,
                 stdout=open('/dev/null', 'w'),
@@ -139,24 +216,24 @@ class Host(models.Model):
 
     @property
     def running_now(self):
-        if datetime.now() + timedelta(seconds=-30) > self.checked_time:
+        if datetime.now() + timedelta(seconds=-60) > self.checked_time:
             self.check_running()
         return self.running
 
-    def check_running(self, verbosity=0):
-        '''
+    def check_running(self):
+        """
         Uses the hosts data and one of the associated accounts to check the PBS
         queue on the Host. If it has been checked in the last 2 minutes, it
         will return the previously returned result.
 
-        '''
+        """
         self.checked_time = datetime.now()
         if self.state < 0:
             self.running = {}
             self.save()
             return
         account = random.choice(self.accounts)
-        raw_data = account.execute(self.check_queue, verbosity=verbosity)
+        raw_data = account.execute(self.check_queue)
         running = {}
         if not raw_data:
             return
@@ -182,15 +259,51 @@ class Host(models.Model):
         else:
             return {}
 
+    @property
+    def utilization_by_project(self):
+        utilization = defaultdict(int)
+        for job in self.jobs:
+            projects = job.task.project_set.all()
+            for p in projects:
+                utilization[str(p.name)] += float(job.ncpus)/len(projects)
+        if self.ppn*self.nodes > sum(utilization.values()):
+            utilization["Idle"] = self.ppn*self.nodes - sum(utilization.values())
+        return utilization
+
+    @property
+    def utilization_json(self):
+        series = []
+        for k, v in self.utilization_by_project.items():
+            series.append({'data':v, 'label':k})
+        return json.dumps(series)
+
+    @property
+    def ncpus(self):
+        return self.ppn * self.nodes
+
 #===============================================================================#
 
 class Account(models.Model):
+    """ 
+    Base class for a `User` account on a `Host`. 
+
+    Attributes:
+        | host
+        | id
+        | job
+        | run_path
+        | state
+        | user
+        | username
+
+    """
+
     user = models.ForeignKey(User)
     host = models.ForeignKey(Host)
-    username = models.CharField(max_length=20)
-    run_path = models.CharField(max_length=120)
-
+    username = models.CharField(max_length=255)
+    run_path = models.TextField()
     state = models.IntegerField(default=1)
+
     class Meta:
         app_label = 'qmpy'
         db_table = 'accounts'
@@ -199,6 +312,13 @@ class Account(models.Model):
         return '{user}@{host}'.format(user=self.user.username, 
                 host=self.host.name)
 
+    @classmethod
+    def get(cls, user, host):
+        try:
+            return Account.objects.get(user=user, host=host)
+        except cls.DoesNotExist:
+            return Account(host=host, user=user)
+
     @property
     def active(self):
         if self.state < 1:
@@ -208,48 +328,39 @@ class Account(models.Model):
         else:
             return True
 
-    def submit(self, path=None, run_path=None, 
-            qfile=None, verbosity=0):
-        self.execute('mkdir %s' % run_path, verbosity=verbosity,
-                ignore_output=True)
-        self.copy(folder=path, file='*',
-                destination=run_path,
-                verbosity=verbosity)
+    def submit(self, path=None, run_path=None, qfile=None):
+        self.execute('mkdir %s' % run_path, ignore_output=True)
+        self.copy(folder=path, file='*', destination=run_path)
         cmd = 'cd {path} && {sub} {qfile}'.format(
                 path=run_path, 
                 sub=self.host.sub_script,
                 qfile=qfile)
-        stdout = self.execute(cmd, verbosity=verbosity)
+        stdout = self.execute(cmd)
         jid = int(stdout.split()[0].split('.')[0])
-        if self.host.name == 'quest':
-            time.sleep(60)
         return jid
 
-    def execute(self, command='exit 0', verbosity=0, ignore_output=False):
+    def execute(self, command='exit 0', ignore_output=False):
         ssh = 'ssh {user}@{host} "{cmd}"'.format(
                 user=self.username,
                 host=self.host.ip_address,
                 cmd=command)
 
-        if verbosity > 0:
-            print ssh
+        logging.debug(ssh)
 
         call = subprocess.Popen(ssh, shell=True,
                 stdout=subprocess.PIPE,
                 stderr=subprocess.PIPE)
         stdout,stderr = call.communicate()
 
-        if verbosity > 0:
-            print 'stdout:', stdout
-            print 'stderr:', stderr
+        logging.debug('stdout: %s', stdout)
+        logging.debug('stderr: %s', stderr)
         if stderr and not ignore_output:
-            print 'WARNING:'
-            print stderr
+            logging.warn('WARNING: %s', stderr)
         return stdout
 
     def copy(self, destination=None, to=None, # where to send the stuff
             fr=None, file=None, folder=None, # what to send
-            clear_dest_dir=False, verbosity=0, move=False): # some conditions on sending it
+            clear_dest_dir=False, move=False): # some conditions on sending it
 
         if destination is None:
             destination = self.run_path
@@ -263,11 +374,10 @@ class Account(models.Model):
 
         assert (isinstance(to, Account) or to == 'local')
         assert (isinstance(fr, Account) or fr == 'local')
+        assert ( not (file is None and folder is None) )
 
         send_dir = False
-        if file is None and folder is None:
-            print 'Must specify something to copy'
-        elif file is None:
+        if file is None:
             send_dir = True
         elif folder is None:
             folder = os.path.dirname(file)
@@ -281,8 +391,7 @@ class Account(models.Model):
                 stdout, stderr = command.communicate()
             else:
                 stdout, stderr = self.execute('rm -f %/*' % destination)
-            if verbosity:
-                print 'output:', stdout
+            logging.debug('stdout: %s', stdout)
             
         if fr == 'local':
             scp = 'scp '
@@ -306,16 +415,14 @@ class Account(models.Model):
                 user=to.username, host=to.host.ip_address, 
                 path=os.path.abspath(destination))
 
-        if verbosity > 0:
-            print 'copy command:', scp
+        logging.debug('copy command: %s', scp)
         cmd = subprocess.Popen(scp,
                 shell=True,
                 stdout=subprocess.PIPE,
                 stderr=subprocess.PIPE)
         stdout, stderr = cmd.communicate()
-        if verbosity:
-            print 'stdout:', stdout
-            print 'stderr:', stderr
+        logging.debug('stdout: %s', stdout)
+        logging.debug('stderr: %s', stderr)
 
         if move:
             if send_dir:
@@ -323,20 +430,31 @@ class Account(models.Model):
             else:
                 rmcmd = 'rm -f {path}/{file}'.format(file=file,
                         path=os.path.abspath(folder))
-            if verbosity > 0:
-                print 'wiping source: ', rmcmd
+            logging.debug('wiping source: %s', rmcmd)
             stdout = fr.execute(rmcmd)
-            if verbosity > 0:
-                print 'output:', stdout
+            logging.debug('output: %s', stdout)
 
 #===============================================================================#
 
 class Allocation(models.Model):
-    name = models.CharField(max_length=50, db_index=True)
-    key = models.CharField(max_length=100, default='', blank=True, null=True)
+    """
+    Base class for an Allocation on a computing resources.
 
-    host = models.ForeignKey(Host, null=True)
-    users = models.ManyToManyField(User, null=True)
+    Attributes:
+        | host
+        | job
+        | key
+        | name
+        | project
+        | state
+        | users
+
+    """
+    name = models.CharField(max_length=63, primary_key=True)
+    key = models.CharField(max_length=100, default='')
+
+    host = models.ForeignKey(Host)
+    users = models.ManyToManyField(User)
     state = models.IntegerField(default=1)
     
     class Meta:
@@ -370,6 +488,13 @@ class Allocation(models.Model):
         alloc.key=key
         alloc.save()
 
+    @classmethod
+    def get(cls, name):
+        try:
+            return cls.objects.get(name=name)
+        except cls.DoesNotExist:
+            return cls(name=name)
+
     @property
     def active(self):
         if self.state < 1:
@@ -384,7 +509,7 @@ class Allocation(models.Model):
 
     def get_account(self, users=None):
         if users is None:
-            users = self.users.filter(state=1).all()
+            users = self.users.all()
         user = random.choice(list(users))
         return user.account_set.get(host=self.host)
 
@@ -395,7 +520,20 @@ class Allocation(models.Model):
 #===============================================================================#
 
 class Project(models.Model):
-    name = models.CharField(max_length=60)
+    """
+    Base class for a project within qmpy. 
+
+    Attributes:
+        | allocations
+        | entry
+        | name
+        | priority
+        | state
+        | task
+        | users
+
+    """
+    name = models.CharField(max_length=63, primary_key=True)
     priority = models.IntegerField(default=0)
 
     users = models.ManyToManyField(User)
@@ -412,10 +550,10 @@ class Project(models.Model):
     def get(cls, name):
         if isinstance(name, cls):
             return name
-        proj, new = cls.objects.get_or_create(name=name)
-        if new:
-            proj.save()
-        return proj
+        try:
+            return cls.objects.get(name=name)
+        except cls.DoesNotExist:
+            return cls(name=name)
 
     @property
     def completed(self):
@@ -423,8 +561,15 @@ class Project(models.Model):
 
     @property
     def running(self):
-        from queue import Job
-        return Job.objects.filter(state=1, task__project_set=self)
+        return self.task_set.filter(state=1)
+
+    @property
+    def waiting(self):
+        return self.task_set.filter(state=0).order_by('priority')
+
+    @property
+    def failed(self):
+        return self.task_set.filter(state=-1)
 
     @classmethod
     def interactive_create(cls):
@@ -453,10 +598,6 @@ class Project(models.Model):
     def active(self):
         if self.state < 0:
             return False
-        elif not any( a.active for a in self.allocations.all() ):
-            self.state = 0
-            self.save()
-            return False
         else:
             if self.state != 1:
                 self.state = 1
