@@ -1,5 +1,6 @@
 import yaml
 import os, os.path
+from inputs import *
 from qmpy import INSTALL_PATH
 
 vs_path = os.path.dirname(os.path.abspath(__file__))
@@ -18,7 +19,3 @@ for setting, data in thubbards.items():
 HUBBARDS = hubbards
 POTENTIALS = yaml.load(open(vs_path+'/potentials.yml').read())
 
-VASP_SETTINGS = {}
-for f in os.listdir(vs_path+'/inputs/'):
-    settings =  yaml.load(open('%s/inputs/%s' % (vs_path, f)).read())
-    VASP_SETTINGS[f.replace('.yml','')] = settings
