@@ -14,15 +14,17 @@ MANAGERS = ADMINS
 
 DATABASES = {
     'default': {
-        'ENGINE': 'django.db.backends.mysql', # Add 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
-        'NAME': 'qmdb_prod',                      # Or path to database file if using sqlite3.
-        'USER': 'sjk648',                      # Not used with sqlite3.
-        'PASSWORD': '',                  # Not used with sqlite3.
-        'HOST': '',                      # Set to empty string for localhost. Not used with sqlite3.
-        'PORT': ''                       # Set to empty string for default. Not used with sqlite3.
+        'ENGINE': 'django.db.backends.mysql', 
+        'NAME': 'qmdb_dev', 
+        'USER': 'sjk648',                      
+        'PASSWORD': '',                  
+        'HOST': '',                      
+        'PORT': ''                       
     }
 }
 
+ALLOWED_HOSTS = ['www.oqmd.org', 'larue.northwestern.edu']
+
 # Local time zone for this installation. Choices can be found here:
 # http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
 # although not all choices may be available on all operating systems.
@@ -119,6 +121,7 @@ INSTALLED_APPS = (
     'django_extensions',
     'django.contrib.admin',
     'django.contrib.admindocs',
+    'google_analytics',
     'qmpy',
 )
 
@@ -168,3 +171,5 @@ TEMPLATE_CONTEXT_PROCESSORS = (
 #            'LOCATION': 'unix:/var/run/memcached/memcached.pid',
 #            }
 #        }
+
+AUTH_USER_MODEL = 'qmpy.User'
