@@ -3,6 +3,7 @@
 
 from math import *
 from strings import *
+from rendering import *
 from daemon import Daemon
 
 def assign_type(string):
@@ -54,3 +55,15 @@ def atom_sort(atoms):
     if len(comps) == 0:
         return atoms
     return atoms[np.lexsort(comps)].tolist()
+
+def get_docstring(model):
+    names = model._meta.get_all_field_names()
+    longest = max(map(len, names))
+
+    print "+-" + '-'*longest + '-+'
+    for n in names:
+        print '| '+ n.ljust(longest) + ' |'
+        print "+-" + '-'*longest + '-+'
+
+def get_field_list(model):
+    print ', '.join(model._meta.get_all_field_names())
